<?php
/*
	* functions.php
*/

add_action( 'plugins_loaded', 'pm_social_encore_load_textdomain' );
function pm_social_encore_load_textdomain() {
    load_plugin_textdomain( 'pm-social-encore', false, dirname( plugin_basename( __DIR__ ) ) . '/languages' );
}

/**
 * Flush rewrite rules for custom post type archive on single activation
 *
 * @since    0.0.1
 */
function pm_social_encore_install() {
	pm_social_encore_post_type_init();
	flush_rewrite_rules();
	pm_social_encore_database();
}

function pm_social_encore_deactivate() {

}

add_action( 'init', 'pm_social_encore_post_type_init' );

/*
	Social Encore Create Database
*/

function pm_social_encore_database() {
	global $wpdb;

		$table_name = $wpdb->prefix . 'pm_se_logs';

		$sql = "CREATE TABLE $table_name (
			id mediumint(9) NOT NULL AUTO_INCREMENT,
			sid bigint(20) NOT NULL,
			postid bigint(20) NOT NULL,
			social varchar(20) NOT NULL,
			status varchar(200) NOT NULL,
			time datetime DEFAULT '0000-00-00 00:00:00' NOT NULL,
			UNIQUE KEY id (id)
		);";

		require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
		dbDelta( $sql );
}

/*
	* pm_social_encore_query_logs
*/

function pm_social_encore_query_logs($post_id, $social = null) {
	global $wpdb;
	$table_name = $wpdb->prefix . 'pm_se_logs';
	
	if($social) {
	$results = $wpdb->get_results( $wpdb->prepare( 
		"
			SELECT *
			FROM {$table_name} 
			WHERE sid = '%s' AND 
			social LIKE '%s' 
			ORDER BY time DESC
		", 
		$post_id,
		$social
	),OBJECT );
	} else {
	$results = $wpdb->get_results( $wpdb->prepare( 
		"
			SELECT *
			FROM {$table_name} 
			WHERE sid = '%s' 
			ORDER BY time DESC
		", 
		$post_id
	),OBJECT );
	}
	
	return $results;
	
}

/*
	* pm_social_encore_query_log_id
*/

function pm_social_encore_query_log_id() {
	global $wpdb;
	$table_name = $wpdb->prefix . 'pm_se_logs';
	$array = array();

	$results = $wpdb->get_results(
		"
			SELECT DISTINCT postid 
			FROM {$table_name} 
			ORDER BY time DESC
		",OBJECT 
	);
	foreach($results as $result) {
		$array[] = $result->postid;
	}
	return $array;
}

function pm_social_encore_post_types() {
	$array = array(
		'post' => 'Post',
		'page' => 'Page'
	);
	
	return $array;
}

/*
	Get Excerpt
*/
function pm_social_encore_get_excerpt_by_id($post_id, $count){
    $the_post = get_post($post_id); //Gets post ID
    $the_excerpt = $the_post->post_content; //Gets post_content to be used as a basis for the excerpt
    $excerpt_length = $count; //Sets excerpt length by word count
    $the_excerpt = strip_tags(strip_shortcodes($the_excerpt)); //Strips tags and images
    $words = explode(' ', $the_excerpt, $excerpt_length + 1);

    if(count($words) > $excerpt_length) :
        array_pop($words);
        array_push($words, '…');
        $the_excerpt = implode(' ', $words);
    endif;

    return $the_excerpt;
}

/*
	Get Settings URL
*/

function pm_social_encore_settings_url() {
	$url = admin_url('admin.php?page=pm_social_encore_settings');
	return $url;
}

/*
	* pm_social_encore_settings_fields
	* Fields for Settings
*/
function pm_social_encore_setting_fields(){
	$fields = array(
	'facebook' => array(
		'social_name' => __( 'Facebook', 'pm-social-encore' ),
		'general_description' => __( 'App Domains should be your sitedomain.com', 'pm-social-encore' ),
		'fields' => array(
			'app_id' => array(
				'name' => 'app_id',
				'description' => __( 'Enter your Facebook AppID', 'pm-social-encore' ),
				'field' => 'input',
				'type' => 'text',
				'label' => __( 'App Id', 'pm-social-encore' ),
				'default' => '',
				'repeatable' => false,
				'show_add' => false,
				'required' => true
			),
			'app_secret' => array(
				'name' => 'app_secret',
				'description' => __( 'Enter your Facebook Secret Key', 'pm-social-encore' ),
				'field' => 'input',
				'type' => 'text',
				'label' => __( 'App Secret', 'pm-social-encore' ),
				'default' => '',
				'repeatable' => false,
				'show_add' => false,
				'required' => true
			),
			'url' => array(
				'name' => 'url',
				'description' => __( 'Enter your Facebook Profile, Facebook Page, Facebook Group', 'pm-social-encore' ),
				'field' => 'input',
				'type' => 'text',
				'label' => __( 'Facebook URL', 'pm-social-encore' ),
				'default' => '',
				'repeatable' => false,
				'show_add' => false,
				'required' => true
			),
			'token' => array(
				'name' => 'token',
				'description' => __( 'Access Token', 'pm-social-encore' ),
				'field' => 'input',
				'type' => 'hidden',
				'label' => __('Access Token', 'pm-social-encore' ),
				'default' => '',
				'repeatable' => false,
				'show_add' => false,
				'required' => false
			),
			'authorized' => array(
				'name' => 'authorized',
				'description' => __( 'If Social Authorized', 'pm-social-encore' ),
				'field' => 'input',
				'type' => 'hidden',
				'label' => __( 'Authorized', 'pm-social-encore' ),
				'default' => 0,
				'repeatable' => false,
				'show_add' => false,
				'required' => false
			)
		)
	),	
	'twitter' => array(
		'social_name' => __( 'Twitter', 'pm-social-encore' ),
		'general_description' => __( 'Callback URL should be your sitedomain.com', 'pm-social-encore' ),
		'fields' => array(
			'app_id' => array(
				'name' => 'app_id',
				'description' => __( 'Enter your Twitter AppID', 'pm-social-encore' ),
				'field' => 'input',
				'type' => 'text',
				'label' => __( 'Api Key', 'pm-social-encore' ),
				'default' => '',
				'repeatable' => false,
				'show_add' => false,
				'required' => true
			),
			'app_secret' => array(
				'name' => 'app_secret',
				'description' => __( 'Enter your Twitter Secret Key', 'pm-social-encore' ),
				'field' => 'input',
				'type' => 'text',
				'label' => __('API Secret', 'pm-social-encore' ),
				'default' => '',
				'repeatable' => false,
				'show_add' => false,
				'required' => true
			),
			'url' => array(
				'name' => 'url',
				'description' => __( 'Enter your Twitter url', 'pm-social-encore' ),
				'field' => 'input',
				'type' => 'text',
				'label' => __('Twitter URL', 'pm-social-encore' ),
				'default' => '',
				'repeatable' => false,
				'show_add' => false,
				'required' => true
			),
			'token' => array(
				'name' => 'token',
				'description' => __( 'Access Token', 'pm-social-encore' ),
				'field' => 'input',
				'type' => 'text',
				'label' => __('Access Token', 'pm-social-encore' ),
				'default' => '',
				'repeatable' => false,
				'show_add' => false,
				'required' => true
			),
			'token_secret' => array(
				'name' => 'token_secret',
				'description' => __( 'Access Token', 'pm-social-encore' ),
				'field' => 'input',
				'type' => 'text',
				'label' => __('Access Token Secret', 'pm-social-encore' ),
				'default' => '',
				'repeatable' => false,
				'show_add' => false,
				'required' => true
			),
			'authorized' => array(
				'name' => 'authorized',
				'description' => __( 'If Social Authorized', 'pm-social-encore' ),
				'field' => 'input',
				'type' => 'hidden',
				'label' => __('Authorized', 'pm-social-encore' ),
				'default' => 0,
				'repeatable' => false,
				'show_add' => false,
				'required' => false
			)
		)
	),
	'linkedin' => array(
		'social_name' => __( 'LinkedIn', 'pm-social-encore' ),
		'general_description' => __( 'App Redirect url is: ', 'pm-social-encore' ). PM_SOCIAL_ENCORE_DIR.'modules/libs/login/linkedin-login.php',
		'fields' => array(
			'app_id' => array(
				'name' => 'app_id',
				'description' => __( 'Enter your LinkedIn AppID', 'pm-social-encore' ),
				'field' => 'input',
				'type' => 'text',
				'label' => __('Api Key', 'pm-social-encore' ),
				'default' => '',
				'repeatable' => false,
				'show_add' => false,
				'required' => true
			),
			'app_secret' => array(
				'name' => 'app_secret',
				'description' => __( 'Enter your LinkedIn Secret Key', 'pm-social-encore' ),
				'field' => 'input',
				'type' => 'text',
				'label' => __('API Secret', 'pm-social-encore' ),
				'default' => '',
				'repeatable' => false,
				'show_add' => false,
				'required' => true
			),
			'token' => array(
				'name' => 'token',
				'description' => __( 'Access Token', 'pm-social-encore' ),
				'field' => 'input',
				'type' => 'hidden',
				'label' => __('Access Token', 'pm-social-encore' ),
				'default' => '',
				'repeatable' => false,
				'show_add' => false,
				'required' => false
			),
			'authorized' => array(
				'name' => 'authorized',
				'description' => __( 'If Social Authorized', 'pm-social-encore' ),
				'field' => 'input',
				'type' => 'hidden',
				'label' => __('Authorized', 'pm-social-encore' ),
				'default' => 0,
				'repeatable' => false,
				'show_add' => false,
				'required' => false
			)
		)
	),	
	);
	
	return $fields;
}

/*
	* pm_pmover_setting_fields
	* Fields for Settings
*/
function pm_social_encore_set_field($attr,$social,$opt){
	$output = '';
	$values = pm_social_encore_setting_fields();
	if($attr == 'name') {
		$output = PM_SOCIAL_ENCORE_SETTINGS_OPTION . "[{$social}]" . "[{$opt}]";
	} elseif($attr == 'id') {
		$output = PM_SOCIAL_ENCORE_SETTINGS_OPTION . "_{$social}" . "_{$opt}";
	} else {
		$output = $values[$social]['fields'][$opt][$attr];
	}
	return $output;
}


/**
 * Register a post type.
 *
 * @link http://codex.wordpress.org/Function_Reference/register_post_type
 */
function pm_social_encore_post_type_init() {
	$labels_social_encore = array(
		'name'               => _x( 'Schedules', 'post type general name', 'pm-social-encore' ),
		'singular_name'      => _x( 'Schedule', 'post type singular name', 'pm-social-encore' ),
		'menu_name'          => _x( 'Encore Schedules', 'admin menu', 'pm-social-encore' ),
		'name_admin_bar'     => _x( 'Schedule', 'add new on admin bar', 'pm-social-encore' ),
		'add_new'            => _x( 'Add New', 'Schedule', 'pm-social-encore' ),
		'add_new_item'       => __( 'Add New Schedule', 'pm-social-encore' ),
		'new_item'           => __( 'New Schedule', 'pm-social-encore' ),
		'edit_item'          => __( 'Edit Schedule', 'pm-social-encore' ),
		'view_item'          => __( 'View Schedule', 'pm-social-encore' ),
		'all_items'          => __( 'All Schedules', 'pm-social-encore' ),
		'search_items'       => __( 'Search Schedules', 'pm-social-encore' ),
		'parent_item_colon'  => __( 'Parent Schedules:', 'pm-social-encore' ),
		'not_found'          => __( 'No Schedules found.', 'pm-social-encore' ),
		'not_found_in_trash' => __( 'No Schedules found in Trash.', 'pm-social-encore' )
	);

	$args_social_encore = array(
		'labels'             => $labels_social_encore,
		'public'             => false,
		'publicly_queryable' => false,
		'exclude_from_search' => true,
		'show_ui'            => true,
		'show_in_menu'       => true,
		'query_var'          => false,

		'capability_type'    => 'post',
		'has_archive'        => false,
		'hierarchical'       => false,
		'menu_position'      => 155,
		'supports'           => array( 'title')
	);


	register_post_type( 'social_encore', $args_social_encore );
}

/*
	* pm_social_encore
	* Register Columns on Schedules
*/

function pm_social_encore_columns_head($defaults) {
	$defaults['pm_social_encore_next_schedule'] = esc_html(__('Next Cron Schedule', 'pm-social-encore'));
	return $defaults;
}

function pm_social_encore_column_content($column_name, $pid) {
	$output ="<div class='social_encore_box_wrapper'>";
	if ($column_name == 'pm_social_encore_next_schedule') {
		$extension = $pid;
		$cron_args = array($pid);
		$timestamp = (wp_next_scheduled( 'pm_social_encore_task_'.$extension,$cron_args ) ? wp_next_scheduled( 'pm_social_encore_task_'.$extension,$cron_args ) : '');
		$output .= ($timestamp ? date_i18n('F j, Y @ H:i', $timestamp, false).' - UTC' : '');
    }
	$output .= "</div>";
	print $output;
}

add_filter('manage_social_encore_posts_columns', 'pm_social_encore_columns_head');
add_action('manage_social_encore_posts_custom_column', 'pm_social_encore_column_content', 10, 2);


/*
	* pm_social_encore_suggestion_vote
*/

function pm_social_encore_add_log($args) {
	global $wpdb;
	$table_name = $wpdb->prefix . 'pm_se_logs';
	
	$defaults = array(
		'sid'	=>	'',
		'postid'	=>	'',
		'social'	=> '',
		'status'	=> ''
	);

	$args = wp_parse_args( $args, $defaults );

	$add = $wpdb->query(
		$wpdb->prepare(
			"INSERT INTO {$table_name} SET
				`sid`    = '%s',
				`postid`    = '%s',
				`social`    = '%s',
				`status`    = '%s',
				`time`    = NOW()
			;",
			absint( $args['sid'] ),
			absint( $args['postid'] ),
			$args['social'],
			$args['status']
		)
	);
}

/*
	* Social Encore Generate array for post_meta 
*/

function pm_social_encore_post_meta_array($post_id) {
	$data = array();
	$data['se_enabled'] = (get_post_meta($post_id, '_pm_se_enabled', true) ? get_post_meta($post_id, '_pm_se_enabled', true) : 'yes');
	$data['se_type'] = (get_post_meta($post_id, '_pm_se_type', true) ? get_post_meta($post_id, '_pm_se_type', true) : 'bet');
	$data['se_post_type'] = (get_post_meta($post_id, '_pm_se_post_type', true) ? get_post_meta($post_id, '_pm_se_post_type', true) : 'post');
	$data['se_date_a'] = (get_post_meta($post_id, '_pm_se_date_a', true) ? get_post_meta($post_id, '_pm_se_date_a', true) : '');
	$data['se_date_b'] = (get_post_meta($post_id, '_pm_se_date_b', true) ? get_post_meta($post_id, '_pm_se_date_b', true) : '');
	$data['se_date_initial'] = (get_post_meta($post_id, '_pm_se_date_initial', true) ? get_post_meta($post_id, '_pm_se_date_initial', true) : '');
	$data['se_interval'] = (get_post_meta($post_id, '_pm_se_interval', true) ? get_post_meta($post_id, '_pm_se_interval', true) : 0);
	$data['se_social'] = (get_post_meta($post_id, '_pm_se_social', true) ? get_post_meta($post_id, '_pm_se_social', true) : array());
	$data['se_specific_post'] = (get_post_meta($post_id, '_pm_se_specific_post', true) ? get_post_meta($post_id, '_pm_se_specific_post', true) : '');
	$data['se_source'] = (get_post_meta($post_id, '_pm_se_source', true) ? get_post_meta($post_id, '_pm_se_source', true) : 'random');
	$data['se_allow_duplicate'] = (get_post_meta($post_id, '_pm_se_allow_duplicate', true) ? get_post_meta($post_id, '_pm_se_allow_duplicate', true) : '');
	

	return $data;
}

/*
	* Print Social Encore Date Between
*/

function pm_social_encore_date_between($data) { 
	$args = array(
		'post_type' => $data['se_post_type'],
		'posts_per_page' => -1,
		'orderby' => 'rand',
		'date_query' => array(
			array(
				'after'     => $data['se_date_a'],
				'before'    => $data['se_date_b'],
				'inclusive' => true,
			),
		),
	);
	
	return $args;
}

/*
	* Print Social Encore Date Before
*/

function pm_social_encore_date_before($data) { 
	$args = array(
		'post_type' => $data['se_post_type'],
		'posts_per_page' => -1,
		'orderby' => 'rand',
		'date_query' => array(
			array(
				'before'    => $data['se_date_a']
			),
		),
	);
	
	return $args;
}

/*
	* Print Social Encore Date After
*/

function pm_social_encore_date_after($data) { 
	$args = array(
		'post_type' => $data['se_post_type'],
		'posts_per_page' => -1,
		'orderby' => 'rand',
		'date_query' => array(
			array(
				'after'     => $data['se_date_a']
			),
		),
	);
	
	return $args;
}

/*
	* Generate Post ID for Random Schedule
*/

function pm_social_encore_share_post_random($data) {
	$post_id = null;
	$args = array();
	$selected = false;
	$post_logs = pm_social_encore_query_log_id();
	$allow_duplicate = $data['se_allow_duplicate'];
	
	if( $data['se_type'] == 'bet') { 
		$args = pm_social_encore_date_between($data);
	} elseif( $data['se_type'] == 'before') {
		$args = pm_social_encore_date_before($data);
	} elseif( $data['se_type'] == 'after') {
		$args = pm_social_encore_date_after($data);
	}
	
	$pm_query = new WP_Query( $args );

	// The Loop
	if ( $pm_query->have_posts() ) {
		while ( $pm_query->have_posts() ) {
			if(!$selected) {
				if($allow_duplicate != 'yes'  && !in_array($pm_query->post->ID, $post_logs) ) {
					$pm_query->the_post();
					$post_id = $pm_query->post->ID;
					$selected = true;
					break;
				} elseif($allow_duplicate == 'yes') {
					$pm_query->the_post();
					$post_id = $pm_query->post->ID;
					$selected = true;
					break;
				}
			}
		}
	} else {
		// no posts found
	}
	/* Restore original Post Data */
	wp_reset_postdata();
	
	return $post_id;
}

/*
	* Sets the Cron Function to Create Cron Schedules
	* Trigger Posting to Different Social Sites
	* Used for pm_social_encore_set_cron()
*/

function pm_social_encore_task_function($post_id) {
	$published_id = '';
	$data = pm_social_encore_post_meta_array($post_id);
	// $post_id is the sid for Database
	// get the Post ID for Posting
	if( $data['se_enabled'] == 'yes') {
		if( $data['se_source'] == 'random') { // Random
			$published_id = pm_social_encore_share_post_random($data);
		} elseif( $data['se_source'] == 'specific' ) { // Specific
			$published_id = $data['se_specific_post'];
		} // end Random or Specific
		
		if($published_id && in_array('facebook', $data['se_social']) ) {
			$facebook = pm_social_encore_facebook_publish($published_id, $post_id);
		}
		if($published_id && in_array('twitter', $data['se_social']) ) {
			$twitter = pm_social_encore_twitter_publish($published_id, $post_id);
		}
		if($published_id && in_array('linkedin', $data['se_social']) ) {
			$linkedin = pm_social_encore_linkedin_publish($published_id, $post_id);
		}
		
		
	}// end enabled
}

/*
	* Sets the Cron Schedules for Created Encores
*/

function pm_social_encore_set_cron() {
	// The Query
	$args = array(
		'post_type' => 'social_encore',
		'posts_per_page' => -1
	);
	$pm_query = new WP_Query( $args );

	// The Loop
	if ( $pm_query->have_posts() ) {
		while ( $pm_query->have_posts() ) {
			$pm_query->the_post();
			$post_id = $pm_query->post->ID;
			
			pm_social_encore_schedule_event($post_id);
		}
	} else {
		// no posts found
	}
	/* Restore original Post Data */
	wp_reset_postdata();

}

add_action('init', 'pm_social_encore_set_cron');

/*
	* Social Encore Set or Remove Schedule
*/

function pm_social_encore_schedule_event($post_id, $type = 'init') {
	$se_interval = get_post_meta($post_id,'_pm_se_interval',true);
	$se_enabled = (get_post_meta($post_id, '_pm_se_enabled', true) ? get_post_meta($post_id, '_pm_se_enabled', true) : 'yes');
	$se_date_initial = (get_post_meta($post_id, '_pm_se_date_initial', true) ? strtotime(get_post_meta($post_id, '_pm_se_date_initial', true), current_time( 'timestamp' )) : time());
	
	$extension = $post_id;
	$interval = $post_id.'_'.$se_interval;
	$cron_args = array($post_id);
	
	if($se_enabled == 'yes') {
		if(!empty($se_interval) || $se_interval > 0 ):
		//pm_social_encore_task_hook
		if ( $type == 'save' && wp_next_scheduled( 'pm_social_encore_task_'.$extension,$cron_args ) ) {
			wp_clear_scheduled_hook( 'pm_social_encore_task_'.$extension, $cron_args );
		}
		if ( !wp_next_scheduled( 'pm_social_encore_task_'.$extension,$cron_args ) ) {
		  wp_schedule_event( $se_date_initial, 'pm_se_'.$interval, 'pm_social_encore_task_'.$extension, $cron_args );
		}
		add_action( 'pm_social_encore_task_'.$extension, 'pm_social_encore_task_function');
		endif;
	} 
	if($se_enabled == 'no') {
		if ( wp_next_scheduled( 'pm_social_encore_task_'.$extension,$cron_args ) ) {
			wp_clear_scheduled_hook( 'pm_social_encore_task_'.$extension, $cron_args );
		}
	}
}

/*
	* Social Encore Add Interval List of Schedules
*/

add_filter( 'cron_schedules', 'pm_social_encore_add_schedule' ); 
	
function pm_social_encore_add_schedule( $schedules ) {
	// The Query
	$args = array(
		'post_type' => 'social_encore',
		'posts_per_page' => -1
	);
	$pm_query = new WP_Query( $args );

	// The Loop
	if ( $pm_query->have_posts() ) {
		while ( $pm_query->have_posts() ) {
			$pm_query->the_post();
			$post_id = $pm_query->post->ID;
			$se_interval = get_post_meta($post_id,'_pm_se_interval',true);
			
			if(!empty($se_interval) ):
			$schedules['pm_se_'.$post_id.'_'.$se_interval] = array(
				'interval' => 60 * $se_interval, //7 days * 24 hours * 60 minutes * 60 seconds
				'display' => __( $se_interval.' Minutes', 'pm-social-encore' )
			);
			endif;
		}
	} else {
		// no posts found
	}
	/* Restore original Post Data */
	wp_reset_postdata();

  
  return $schedules;
}

/*
	* Facebook Initialize
*/

function pm_social_encore_facebook_initialize() {
	require_once('libs/facebook/autoload.php');
	$setting_fields = pm_social_encore_setting_fields();
	$options = get_option(PM_SOCIAL_ENCORE_SETTINGS_OPTION, array());
	$sn = 'facebook';
	$creds = (isset($options[$sn]) ? $options[$sn] : '');
	$error = true;
	$fb = '';
	
	if(is_array($creds)) {
		foreach($creds as $key => $cred) {
			if($setting_fields[$sn]['fields'][$key]['required']) {
			$error = (empty($cred) ? true : false );
			if($error) {break;}
			}
		}
	}
	
	if($error){
		//Variables to validate
	} else {
		$_SESSION[$sn.'_app_id'] = (string) $creds['app_id'];
		$_SESSION[$sn.'_app_secret'] = (string) $creds['app_secret'];
		$fb_user_tokken = (isset($creds['token']) ? $creds['token'] : '');
		if(empty($fb_user_tokken) && isset($_SESSION[$sn.'_access_token']) && !empty($_SESSION[$sn.'_access_token'])) {
			
			$options[$sn]['token'] = $_SESSION[$sn.'_access_token'];
			$options[$sn]['authorized'] = 1;
			update_option(PM_SOCIAL_ENCORE_SETTINGS_OPTION, $options);
			$fb_user_tokken = $_SESSION['facebook_access_token'];
			unset($_SESSION['facebook_access_token']);
		}
		
		$fb = new Facebook\Facebook([
		  'app_id' => $creds['app_id'],
		  'app_secret' => $creds['app_secret'],
		  'default_graph_version' => 'v2.4',
		  'default_access_token' => (isset($fb_user_tokken) ? $fb_user_tokken : '')
		]);
	}
	return $fb;
}

/*
	* Facebook Connect
*/

function pm_social_encore_facebook_connect() {
	$fb = pm_social_encore_facebook_initialize();
	
	$setting_fields = pm_social_encore_setting_fields();
	$options = get_option(PM_SOCIAL_ENCORE_SETTINGS_OPTION, array());
	$sn = 'facebook';
	$creds = (isset($options[$sn]) ? $options[$sn] : '');
	
	if($fb):
		try {
			$current_user = $fb->get('/me');
			$user = $current_user->getGraphUser();
			?>
			<h3><?php print $setting_fields[$sn]['social_name']; ?></h3>
			<p><strong><?php esc_html(_e( 'Connected as: ', 'pm-social-encore' )); ?></strong> <?php print $user['name']; ?></p>
			<p><strong><?php esc_html(_e( 'Posting at: ', 'pm-social-encore' )); ?></strong> <?php print $creds['url']; ?></p>
			<?php
		} catch(Facebook\Exceptions\FacebookResponseException $e) {
		//echo 'Graph returned an error: ' . $e->getMessage();
			$helper = $fb->getRedirectLoginHelper();
			$permissions = ['manage_pages','publish_actions','publish_pages','user_managed_groups']; // optional
			$loginUrl = $helper->getLoginUrl(PM_SOCIAL_ENCORE_DIR.'modules/libs/login/facebook-login.php', $permissions);
			$image_url = PM_SOCIAL_ENCORE_DIR.'modules/src/images/login-'.$sn.'.png';
			echo '<p class="pm_social_encore_login_facebook"><a href="' . $loginUrl . '"><img src="'.$image_url.'" height="30" /></a></p>';
		} catch(Facebook\Exceptions\FacebookSDKException $e) {
			echo esc_html(sprintf(__('Facebook SDK returned an error: %s', 'pm-social-encore'),$e->getMessage()));
		}
	endif; //End $fb
}

/*
	* Facebook Logout
*/

function pm_social_encore_facebook_logout() {
	$sn = 'facebook';
	$logoutUrl = $_SERVER['REQUEST_URI']."social={$sn}&logout=yes";
	$settings_url = pm_social_encore_settings_url();
	if(isset($_GET['logout']) && $_GET['logout'] == 'yes' && $_GET['social'] == $sn) {
	//$fb->get('/me/permissions','DELETE');
	$options = get_option(PM_SOCIAL_ENCORE_SETTINGS_OPTION, array());
	$options[$sn]['token'] = '';
	$options[$sn]['authorized'] = 0;
	update_option(PM_SOCIAL_ENCORE_SETTINGS_OPTION, $options);
	print $admin_url;
	echo "<script> window.location='{$settings_url}'; </script> ";
	}
	
	echo '<p><a href="' . $logoutUrl . '">Logout Account!</a></p>';
}

/*
	* Facebook Post
*/

function pm_social_encore_facebook_publish($post_id, $sid) { 
	$post_title = get_the_title($post_id);
	$post_permalink = get_permalink($post_id);
	
	$fb = pm_social_encore_facebook_initialize();
	
	$setting_fields = pm_social_encore_setting_fields();
	$options = get_option(PM_SOCIAL_ENCORE_SETTINGS_OPTION, array());
	$sn = 'facebook';
	$creds = (isset($options[$sn]) ? $options[$sn] : '');
	
	$message = '';
	if($fb):
	//Start
	$linkData = [
		'link' => $post_permalink,
		'message' => '',
	];
	try {
		$response = $fb->get("/{$creds['url']}?fields=access_token");
		$user = $response->getGraphUser();
		//$userResponse = $fb->post("/{$creds['url']}/feed", $linkData, $user['access_token']);
	} catch(Facebook\Exceptions\FacebookResponseException $e) {
		$message =  esc_html(sprintf(__('Graph returned an error: %s', 'pm-social-encore'),$e->getMessage()));
	} catch(Facebook\Exceptions\FacebookSDKException $e) {
		$message = esc_html(sprintf(__('Facebook SDK returned an error: %s', 'pm-social-encore'),$e->getMessage()));
	}
	if(!isset($user)) {
		try {
			$response = $fb->get("/{$creds['url']}");
			$user = $response->getGraphUser();
			//$userResponse = $fb->post("/{$creds['url']}/feed", $linkData, $user['access_token']);
		} catch(Facebook\Exceptions\FacebookResponseException $e) {
			$message =  esc_html(sprintf(__('Graph returned an error: %s', 'pm-social-encore'),$e->getMessage()));
		} catch(Facebook\Exceptions\FacebookSDKException $e) {
			$message = esc_html(sprintf(__('Facebook SDK returned an error: %s', 'pm-social-encore'),$e->getMessage()));
		}
	}	

	//End
	try {
		$userResponse = $fb->post("/{$creds['url']}/feed", $linkData, $user['access_token']);
		$graphNode = $userResponse->getGraphNode();
		$message =  esc_html(sprintf(__('Success id: %s', 'pm-social-encore'), $graphNode['id'] ));
	} catch(Facebook\Exceptions\FacebookResponseException $e) {
		$message =  esc_html(sprintf(__('Graph returned an error: %s', 'pm-social-encore'),$e->getMessage()));
		//exit;
	} catch(Facebook\Exceptions\FacebookSDKException $e) {
		$message = esc_html(sprintf(__('Facebook SDK returned an error: %s', 'pm-social-encore'),$e->getMessage()));
	//exit;
	}
	endif; //End $fb
	
	$log = array(
		'sid'	=>	$sid,
		'postid'	=>	$post_id,
		'social'	=> 'facebook',
		'status'	=> $message
	);
	pm_social_encore_add_log($log);
	
	return $message;
}

/*
	* Twitter Post
*/

function pm_social_encore_twitter_publish($post_id, $sid) { 
	$post_title = get_the_title($post_id);
	$post_permalink = get_permalink($post_id);
	
	$setting_fields = pm_social_encore_setting_fields();
	$options = get_option(PM_SOCIAL_ENCORE_SETTINGS_OPTION, array());
	$sn = 'twitter';
	$creds = (isset($options[$sn]) ? $options[$sn] : '');

	$message = '';

	require_once('libs/twitter/codebird.php');
	\Codebird\Codebird::setConsumerKey($creds['app_id'], $creds['app_secret']);
	$tw = \Codebird\Codebird::getInstance();
 	$tw->setToken($creds['token'], $creds['token_secret']);

	$params = array(
		'status' => $post_title . ' ' . $post_permalink
	);
	$reply = $tw->statuses_update($params);
	
	if($reply->id) {
		$message =  esc_html(sprintf(__('Success id: %s', 'pm-social-encore'),$reply->id ));
	} else {
		$error = $reply->errors;
		$message =  esc_html(sprintf(__('Error: %s', 'pm-social-encore'),$error[0]->message ));
	}
	print $message;

	
	$log = array(
		'sid'	=>	$sid,
		'postid'	=>	$post_id,
		'social'	=> 'twitter',
		'status'	=> $message
	);
	pm_social_encore_add_log($log);

	//return $message;
}

/*
	* LinkedIn Initialize
*/

function pm_social_encore_linkedin_initialize() {
	$setting_fields = pm_social_encore_setting_fields();
	$options = get_option(PM_SOCIAL_ENCORE_SETTINGS_OPTION, array());
	$sn = 'linkedin';
	$creds = (isset($options[$sn]) ? $options[$sn] : '');
	$error = true;
	$sc = '';
	
	if(is_array($creds)) {
		foreach($creds as $key => $cred) {
			if($setting_fields[$sn]['fields'][$key]['required']) {
			$error = (empty($cred) ? true : false );
			if($error) {break;}
			}
		}
	}
	
	if($error){
		//Variables to validate
		$sc = 'error';
	} else {
		$_SESSION[$sn.'_app_id'] = (string) $creds['app_id'];
		$_SESSION[$sn.'_app_secret'] = (string) $creds['app_secret'];
		
		$user_tokken = (isset($creds['token']) ? $creds['token'] : '');
		if(empty($user_tokken) && isset($_SESSION[$sn.'_access_token']) && !empty($_SESSION[$sn.'_access_token'])) {
			
			$options[$sn]['token'] = $_SESSION[$sn.'_access_token'];
			$options[$sn]['authorized'] = 1;
			update_option(PM_SOCIAL_ENCORE_SETTINGS_OPTION, $options);
			$user_tokken = $_SESSION[$sn.'_access_token'];
			unset($_SESSION[$sn.'_access_token']);
		}
		$sc = $user_tokken;
		
	}
	return $sc;
}

/*
	* LinkedIn Connect
*/

function pm_social_encore_linkedin_connect() {
	$li = pm_social_encore_linkedin_initialize();
	
	$setting_fields = pm_social_encore_setting_fields();
	$options = get_option(PM_SOCIAL_ENCORE_SETTINGS_OPTION, array());
	$sn = 'linkedin';
	$creds = (isset($options[$sn]) ? $options[$sn] : '');
	
	
	if($li) {
		//Do when Logged In
	} else {
		$loginUrl = $_SESSION['pm_social_encore_linkedin_login'];
		$image_url = PM_SOCIAL_ENCORE_DIR.'modules/src/images/login-'.$sn.'.png';
		echo '<p class="pm_social_encore_login_linkedin"><a href="' . $loginUrl . '"><img src="'.$image_url.'" height="30" /></a></p>';
	}
}

/*
	* LinkedIn Logout
*/

function pm_social_encore_social_logout($sn) {
	$logoutUrl = $_SERVER['REQUEST_URI']."&social={$sn}&logout=yes";
	$settings_url = pm_social_encore_settings_url();
	if(isset($_GET['logout']) && $_GET['logout'] == 'yes' && $_GET['social'] == $sn) {
	$options = get_option(PM_SOCIAL_ENCORE_SETTINGS_OPTION, array());
	$options[$sn]['token'] = '';
	$options[$sn]['authorized'] = 0;
	update_option(PM_SOCIAL_ENCORE_SETTINGS_OPTION, $options);
	echo "<script> window.location='{$settings_url}'; </script> ";
	}
	echo '<p><a href="' . $logoutUrl . '">'.esc_html(sprintf(__('Logout %s account!', 'pm-social-encore'),$sn)).'</a></p>';
}

function pm_social_encore_linkedin_publish($post_id, $sid) {
	$post_title = get_the_title($post_id);
	$post_permalink = get_permalink($post_id);
    $post_excerpt = pm_social_encore_get_excerpt_by_id($post_id, 20);
	
	$setting_fields = pm_social_encore_setting_fields();
	$options = get_option(PM_SOCIAL_ENCORE_SETTINGS_OPTION, array());
	$sn = 'linkedin';
	$creds = (isset($options[$sn]) ? $options[$sn] : '');
	$message = '';
	
	$service_url = 'https://api.linkedin.com/v1/people/~/shares?format=json';
	$curl = curl_init($service_url);
	$curl_post_data = array(
		'content' => array(
			'title' => $post_title, 
			'description' => $post_excerpt,
			'submitted-url' => $post_permalink
			),
		'visibility' => array('code' => 'anyone')
		);

	curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
	curl_setopt($curl, CURLOPT_POST, true);
	curl_setopt($curl, CURLOPT_HTTPHEADER, array("Content-Type: application/json", "Authorization: Bearer " . $creds['token'], "x-li-format: json"));
	curl_setopt($curl, CURLOPT_POSTFIELDS, json_encode($curl_post_data));
	$curl_response = curl_exec($curl);
	curl_close($curl);
	
	$response = json_decode($curl_response);
		
	if(isset($response->updateKey)) {
		$message =  esc_html(sprintf(__('Success id: %s', 'pm-social-encore'), $response->updateKey ));
	} else {
		$message =  esc_html(sprintf(__('Error: %s', 'pm-social-encore'), $response->message ));
	}
	
	$log = array(
		'sid'	=>	$sid,
		'postid'	=>	$post_id,
		'social'	=> $sn,
		'status'	=> $message
	);
	
	pm_social_encore_add_log($log);

	return $message;
}
